#pragma once

#include "Agent.h"
#include "AgentStates.h"
#include "Messages.h"
#include "EntityManager.h"
#include "AgentTeam.h"

Agent::Agent():BaseEntity(0), 
	m_Steering(this), 
	m_StateMachine(this, AgentState_Idle::Instance(), AgentState_GlobalRed::Instance(), nullptr),
	m_Health(100.0f),
	m_Weapon(this, 1000.0f, 7.0f, 10)
{
	
}

Agent::Agent(int ID):BaseEntity(ID), 
	m_Steering(this), 
	m_StateMachine(this, AgentState_Idle::Instance(), AgentState_GlobalRed::Instance(), nullptr),
	m_Health(100.0f),
	m_Weapon(this, 500.0f, 7.0f, 10)
{
	
}

Agent::~Agent()
{

}

void Agent::Update(float dt)
{
	m_Physics.Update(dt);
	m_StateMachine.Update(dt);
	m_Weapon.Update(dt);
}


void Agent::Render()
{
	m_StateMachine.Render();
	m_Weapon.Render();
}

void Agent::UseWeapon()
{
	m_Weapon.Fire();
}

void Agent::ReceiveMessage(Telegram incoming)
{
	switch(incoming.msg)
	{
	case Message::msg_Move:
		m_StateMachine.ChangeState(AgentState_Move::Instance());
		return;
	case Message::msg_Stop:
		m_StateMachine.ChangeState(AgentState_Idle::Instance());
		return;
	case Message::msg_Retreat:
		m_StateMachine.ChangeState(AgentState_Retreat::Instance());
		return;
	case Message::msg_Attack:
		m_StateMachine.ChangeState(AgentState_Attack::Instance());
		return;
	case Message::msg_Destroy:
		m_Team->RemoveMember(m_ID);
		gEntities->RemoveEntity(this->GetID());
		return;
	}
}