#pragma once
#include "BaseEntity.h"
#include "Physics.h"
#include "Steering.h"		
#include "StateMachine.h"
#include "Rifle.h"

class AgentTeam;

class Agent : public BaseEntity
{
	Rifle					m_Weapon;				//Object that handles weapon firing.
	Physics					m_Physics;				//Object that stores all the physics related data.
	Steering				m_Steering;				//Object that stores all the steering algorithms and related data
	float					m_Health;				//Agent's health
	IDirect3DTexture9*		m_Texture;				//Agent's sprite texture
	StateMachine<Agent>		m_StateMachine;			//Agent's controlling FSM. Uses a global state for rendering and checking health, as the sprite doesn't change between states.
	D3DXVECTOR3				m_TargetPosition;		//Current target to move to.
	AgentTeam*				m_Team;					//Team this agent belongs to.
	
public:
	//Accessors
	float GetHealth(){return m_Health;}
	Physics* GetPhysics(){return &m_Physics;}
	StateMachine<Agent>* GetFSM(){return &m_StateMachine;}
	Steering* GetSteering(){return &m_Steering;}
	D3DXVECTOR3 GetTargetPosition(){return m_TargetPosition;}
	Rifle* GetWeapon(){return &m_Weapon;}
	AgentTeam* GetTeam(){return m_Team;}

	//Mutators
	void SetMoveTarget(D3DXVECTOR3 target){m_TargetPosition = target;}
	void SetTeam(AgentTeam* team){m_Team = team;}
	
	//Main Functions
	void Update(float dt);
	void Render();
	void ReceiveMessage(Telegram incoming);
	void AdjustHealth(float amount){m_Health += amount;}
	void UseWeapon();
	Agent();
	Agent(int ID);
	~Agent();

};