#include "AgentTeam.h"
#include "Messenger.h"
#include "EntityGenerator.h"
#include "EntityManager.h"
#include "D3DXMathFunctions.h"
#include "Defines.h"
#include "Agent.h"
#include "AgentStates.h"
#include "Messages.h"
#include <algorithm>

void AgentTeam::AddMember(int ID)
{
	m_Agents.push_back(ID);
}

void AgentTeam::RemoveMember(int ID)
{
	std::vector<int>::iterator itr = std::find(m_Agents.begin(), m_Agents.end(), ID);

	if(itr != m_Agents.end())
		m_Agents.erase(itr);
}

void AgentTeam::AddAmmoPickup(int ID)
{
	m_AmmoPickups.push_back(ID);
}

void AgentTeam::AddHealthPickup(int ID)
{
	m_HealthPickups.push_back(ID);
}

void AgentTeam::CalculateDOMs()
{
	//Calculate Health Level and set state
	//Level = Average of all agent's health normalized
	float HealthLevel = 0.0f;
	for(unsigned i = 0; i < m_Agents.size(); i++)
	{
		Agent* nextAgent = (Agent*)gEntities->GetEntity(m_Agents[i]);
		HealthLevel += nextAgent->GetHealth();
	}
	HealthLevel /= (100.0f * (float)m_Agents.size());
	//States: Health Low = 0.0->0.5 peak at 0.3, Moderate 0.3->0.8 peak at 0.6, High 0.5->1.0 peak at 0.9 onwards
	float LowDOM = 0.0f;
	float MediumDOM = 0.0f;
	float HighDOM = 0.0f;
	
	//Low DOM
	if(HealthLevel <= 0.5)
	{
		if(HealthLevel <= 0.3)
		{
			LowDOM = HealthLevel / 0.3;
		}
		else

		{
			LowDOM = 1 - (HealthLevel - 0.3) / 0.2;
		}
	}

	//Medium DOM
	if(HealthLevel > 0.3)
	{
		if(HealthLevel <= 0.6)
		{
			MediumDOM = (HealthLevel - 0.3)/0.3;
		}
		else if (HealthLevel <= 0.8)
		{
			MediumDOM = 1 - (HealthLevel - 0.6) / 0.2;
		}
	}

	//High DOM
	if(HealthLevel > 0.5)
	{
		if(HealthLevel < 0.9)
		{
			HighDOM = (HealthLevel - 0.5) / 0.4;
		}
		else
			HighDOM = 1.0;
	}
	if((LowDOM >= MediumDOM) && (LowDOM >= HighDOM))
	{
		m_HealthState = HealthLow;
	}
	else if ((MediumDOM >= HighDOM) && (MediumDOM > LowDOM))
	{
		m_HealthState = HealthModerate;
	}
	else
		m_HealthState = HealthHigh;


	//Calculate Ammo Level and set state
	float AmmoLevel = 0.0f;
	for(unsigned i = 0; i < m_Agents.size(); i++)
	{
		Agent* nextAgent = (Agent*)gEntities->GetEntity(m_Agents[i]);
		AmmoLevel += (float)nextAgent->GetWeapon()->GetAmmoCount()/(float)nextAgent->GetWeapon()->GetMaxAmmo();
	}
	AmmoLevel /= (float)m_Agents.size();
	//States: Ammo Low = 0.0->0.5 peak at 0.3, Moderate 0.3->0.7 peak at 0.6, High 0.5->1.0 peak at 0.9
	LowDOM = 0.0f;
	MediumDOM = 0.0f;
	HighDOM = 0.0f;
	
	//Low DOM
	if(AmmoLevel <= 0.5)
	{
		if(AmmoLevel <= 0.3)
		{
			LowDOM = AmmoLevel / 0.3;
		}
		else

		{
			LowDOM = 1 - (AmmoLevel - 0.3) / 0.2;
		}
	}

	//Medium DOM
	if(AmmoLevel > 0.3)
	{
		if(AmmoLevel <= 0.6)
		{
			MediumDOM = (AmmoLevel - 0.3)/0.3;
		}
		else if (AmmoLevel <= 0.8)
		{
			MediumDOM = 1 - (AmmoLevel - 0.6) / 0.2;
		}
	}

	//High DOM
	if(AmmoLevel > 0.5)
	{
		if(AmmoLevel < 0.9)
		{
			HighDOM = (AmmoLevel - 0.5) / 0.4;
		}
		else
			HighDOM = 1.0;
	}
	if((LowDOM >= MediumDOM) && (LowDOM >= HighDOM))
	{
		m_AmmoState = AmmoLow;
	}
	else if ((MediumDOM >= HighDOM) && (MediumDOM > LowDOM))
	{
		m_AmmoState = AmmoModerate;
	}
	else
		m_AmmoState = AmmoHigh;
}

void AgentTeam::ReceiveMessage(Telegram telegram)
{
	switch(telegram.msg)
	{
	case Message::msg_Attack:
		m_StateMachine.SetCurrentState(AgentTeamState_Attack::Instance());
		return;
	case Message::msg_HealthLow:
		m_StateMachine.SetCurrentState(AgentTeamState_LowHealth::Instance());
		return;
	case Message::msg_AmmoLow:
		m_StateMachine.SetCurrentState(AgentTeamState_LowAmmo::Instance());
		return;
	}

}

AgentTeam::AgentTeam(int ID):BaseEntity(ID), m_StateMachine(this, AgentTeamState_Regroup::Instance(), nullptr, nullptr)
{
	m_Color = TeamColor::red;
}

void AgentTeam::Init()
{
	float yVal;
	if(m_Color == TeamColor::blue)
		yVal = 600;
	else
		yVal = 150;
	int agentID = gEntGenerate->CreateEntity(ent_Agent);
	AddMember(agentID);
	Agent* agent = (Agent*)gEntities->GetEntity(agentID);
	agent->GetPhysics()->Init(D3DXVECTOR3(900,yVal,0), D3DXZEROVEC3, 32.0, 500, 100);
	agent->GetSteering()->AvoidObstacles(16.0f, 1.0f);
	agent->SetTeam(this);
	if(m_Color == TeamColor::blue)
	{
		agent->GetFSM()->SetGlobalState(AgentState_GlobalBlue::Instance());
	}

	agentID = gEntGenerate->CreateEntity(ent_Agent);
	AddMember(agentID);
	agent = (Agent*)gEntities->GetEntity(agentID);
	agent->GetPhysics()->Init(D3DXVECTOR3(500,yVal,0), D3DXZEROVEC3, 32.0, 500, 100);
	agent->GetSteering()->AvoidObstacles(16.0f, 1.0f);
	agent->SetTeam(this);
	if(m_Color == TeamColor::blue)
	{
		agent->GetFSM()->SetGlobalState(AgentState_GlobalBlue::Instance());
	}

	agentID = gEntGenerate->CreateEntity(ent_Agent);
	AddMember(agentID);
	agent = (Agent*)gEntities->GetEntity(agentID);
	agent->GetPhysics()->Init(D3DXVECTOR3(700,yVal,0), D3DXZEROVEC3, 32.0, 500, 100);
	agent->GetSteering()->AvoidObstacles(16.0f, 1.0f);
	agent->SetTeam(this);
	if(m_Color == TeamColor::blue)
	{
		agent->GetFSM()->SetGlobalState(AgentState_GlobalBlue::Instance());
	}
	m_StateMachine.SetGlobalState(AgentTeamState_Global::Instance());
}