#pragma once
#include <vector>
#include "d3dx9.h"
#include "StateMachine.h"
#include "BaseEntity.h"
#include "AgentTeamStates.h"

struct Telegram;
class Agent;

enum TeamColor{ red, blue };
//Fuzzy states in an enumerated fashion. To be set after defuzzification.
enum HealthState{ HealthLow, HealthModerate, HealthHigh };
enum AmmoState{ AmmoLow, AmmoModerate, AmmoHigh };

class AgentTeam : public BaseEntity
{
	std::vector<int>			m_Agents;

	//Agent perception variables.
	float						m_AgentDetectionRange;
	int							m_EnemyTeam;
	std::vector<int>			m_Enemys;
	std::vector<int>			m_HealthPickups;
	std::vector<int>			m_AmmoPickups;
	TeamColor					m_Color;
	AgentTeam*					m_Opponent;
		
	//Current states for fuzzy logic
	HealthState m_HealthState;
	AmmoState m_AmmoState;

	//Calculates the average DOM of the team and adjusts the fuzzy states as necessary.
	void CalculateDOMs();
	
	//State machine for agent team. Uses a global state to handle fuzzy transitions.
	StateMachine<AgentTeam>	m_StateMachine;
public:
	AgentTeam(int ID);
	
	//Accessors
	const std::vector<int> GetMembers(){return m_Agents; }
	const std::vector<int> GetHealthPickups(){return m_HealthPickups; }
	const std::vector<int> GetAmmoPickups(){return m_AmmoPickups; }
	AgentTeam* GetOpponent(){return m_Opponent;}
	AmmoState GetAmmoState(){return m_AmmoState;}
	HealthState GetHealthState(){return m_HealthState;}

	//Modifiers
	void AddMember(int ID);
	void RemoveMember(int ID);
	void AddAmmoPickup(int ID);
	void AddHealthPickup(int ID);
	void SetColor(TeamColor color){m_Color = color;}
	void SetOpponent(AgentTeam* opponent){m_Opponent = opponent;}

	void Update(float dt)
	{
		m_StateMachine.Update(dt); 
		CalculateDOMs();
	}

	void Render(){m_StateMachine.Render();}
	void ReceiveMessage(Telegram incoming);
	void Init();

};