#include "AgentTeamStates.h"
#include "DirectInput.h"
#include "Defines.h"
#include "Agent.h"
#include "EntityManager.h"
#include "Messages.h"
#include "EntityGenerator.h"
#include "D3DXMathFunctions.h"
#include "AgentTeam.h"
#include "Pickups.h"
/*
AgentTeamState_Test* AgentTeamState_Test::Instance()
{
	static AgentTeamState_Test instance;
	return &instance;
}

void AgentTeamState_Test::Enter(AgentTeam* owner)
{

}

void AgentTeamState_Test::Execute(AgentTeam* owner, float dt)
{
	//Check for contact with pickups. Reset pickups and add ammo/health if contacted and usable.
	for(unsigned i = 0; i < owner->GetMembers().size(); i++)
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);

		for(unsigned j = 0; j < owner->GetHealthPickups().size(); j++)
		{
			int nextID = owner->GetHealthPickups()[j];
			HealthPickup* nextPickup = (HealthPickup*)gEntities->GetEntity(nextID);
			float distanceToAgent = D3DXVec3Length(&(agent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
			if(distanceToAgent < (agent->GetPhysics()->GetContactRadius() + nextPickup->GetContactRadius()))
			{
				if(nextPickup->IsActive())
				{
					if(agent->GetHealth() < 100)
					{
						agent->AdjustHealth(nextPickup->GetAmount());
						gMessenger->AddMessage(-1, nextID, msg_Reset, 0.0);
					}
				}
			}

		}
		for(unsigned j = 0; j < owner->GetAmmoPickups().size(); j++)
		{
			int nextID = owner->GetAmmoPickups()[j];
			AmmoPickup* nextPickup = (AmmoPickup*)gEntities->GetEntity(nextID);
			float distanceToAgent = D3DXVec3Length(&(agent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
			if(distanceToAgent < (agent->GetPhysics()->GetContactRadius() + nextPickup->GetContactRadius()))
			{
				if(nextPickup->IsActive())
				{
					if(agent->GetWeapon()->GetAmmoCount() < agent->GetWeapon()->GetMaxAmmo())
					{
						agent->GetWeapon()->Reload(nextPickup->GetAmount());
						gMessenger->AddMessage(-1, nextID, msg_Reset, 0.0);
					}
				}
			}
		}
	}

	if(gInput->KeyDown(DIK_1))
	{
		m_SelectedAgent = 0;
	}
	if(gInput->KeyDown(DIK_2))
	{
		m_SelectedAgent = 1;
	}
	if(gInput->KeyDown(DIK_3))
	{
		m_SelectedAgent = 2;
	}

	if(gInput->KeyReleased(DIK_9))
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[m_SelectedAgent]);
		agent->AdjustHealth(10.0f);
	}

	if(gInput->KeyReleased(DIK_0))
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[m_SelectedAgent]);
		agent->AdjustHealth(-10.0f);
	}

	if(gInput->KeyReleased(DIK_SPACE))
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[m_SelectedAgent]);
		agent->UseWeapon();
	}

	if(gInput->MouseButtonDown(0))
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[m_SelectedAgent]);
		agent->SetMoveTarget(D3DXVECTOR3(gInput->GetMousePos().x, gInput->GetMousePos().y, 0.0f));
		gMessenger->AddMessage(-1, owner->GetMembers()[m_SelectedAgent], Message::msg_Move, 0.0f);
	}

	if(gInput->KeyReleased(DIK_LALT))
	{
		Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[m_SelectedAgent]);
		agent->SetMoveTarget(D3DXVECTOR3(gInput->GetMousePos().x, gInput->GetMousePos().y, 0.0f));
		gMessenger->AddMessage(-1, owner->GetMembers()[m_SelectedAgent], Message::msg_Attack, 0.0f);
	}

	if(gInput->MouseButtonDown(1))
	{
		gMessenger->AddMessage(-1, owner->GetMembers()[m_SelectedAgent], Message::msg_Stop, 0.0f);		
	}

}

void AgentTeamState_Test::Exit(AgentTeam* owner)
{


}

void AgentTeamState_Test::Render(AgentTeam* owner)
{
	//Rendering of agents controlled by entity manager.
}
*/
AgentTeamState_LowHealth* AgentTeamState_LowHealth::Instance()
{
	static AgentTeamState_LowHealth instance;
	return &instance;
}

void AgentTeamState_LowHealth::Enter(AgentTeam* owner)
{
	//Find lowest health agent.
	m_SelectedAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[0]);
	for(unsigned i = 1; i < owner->GetMembers().size(); i++)
	{
		Agent* nextAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
		if(m_SelectedAgent->GetHealth() > nextAgent->GetHealth())
		{
			m_SelectedAgent = nextAgent;
		}
	}
	m_HealthOnEnter = m_SelectedAgent->GetHealth();

	//Find closest health pickup.
	HealthPickup* closestPickup = (HealthPickup*)gEntities->GetEntity(owner->GetHealthPickups()[0]);
	float distanceToClosestPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - closestPickup->GetPosition()));
	for(unsigned i = 1; i < owner->GetHealthPickups().size(); i++)
	{
		HealthPickup* nextPickup = (HealthPickup*)gEntities->GetEntity(owner->GetHealthPickups()[i]);
		float distanceToNextPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
		if((distanceToNextPickup < distanceToClosestPickup) && (nextPickup->IsActive()))
		{
			closestPickup = nextPickup;
			distanceToClosestPickup = distanceToNextPickup;
		}
	}
	m_SelectedPickup = closestPickup;
	//Move agent towards pickup.
	m_SelectedAgent->SetMoveTarget(m_SelectedPickup->GetPosition());
	gMessenger->AddMessage(-1, m_SelectedAgent->GetID(), Message::msg_Move, 0.0f);
}

void AgentTeamState_LowHealth::Execute(AgentTeam* owner, float dt)
{
	//If the agent's health has gone up (by collecting a pickup), restart the state
	if(m_SelectedAgent->GetHealth() > m_HealthOnEnter)
		Enter(owner);

	//If health pickup inactive, find a new pickup and send agent that way.
	if(!m_SelectedPickup->IsActive())
	{
			//Find closest health pickup.
		HealthPickup* closestPickup = (HealthPickup*)gEntities->GetEntity(owner->GetHealthPickups()[0]);
		float distanceToClosestPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - closestPickup->GetPosition()));
		for(unsigned i = 1; i < owner->GetHealthPickups().size(); i++)
		{
			HealthPickup* nextPickup = (HealthPickup*)gEntities->GetEntity(owner->GetHealthPickups()[i]);
			float distanceToNextPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
			if((distanceToNextPickup < distanceToClosestPickup) && (nextPickup->IsActive()))
			{
				closestPickup = nextPickup;
				distanceToClosestPickup = distanceToNextPickup;
			}
		}

		//Move agent towards pickup.
		m_SelectedAgent->SetMoveTarget(m_SelectedPickup->GetPosition());
		gMessenger->AddMessage(-1, m_SelectedAgent->GetID(), Message::msg_Move, 0.0f);
	}


}

void AgentTeamState_LowHealth::Exit(AgentTeam* owner)
{

}

void AgentTeamState_LowHealth::Render(AgentTeam* owner)
{
	//Rendering of agents handled by entity manager.
}

AgentTeamState_LowAmmo* AgentTeamState_LowAmmo::Instance()
{
	static AgentTeamState_LowAmmo instance;
	return &instance;
}

void AgentTeamState_LowAmmo::Enter(AgentTeam* owner)
{
		//Find lowest Ammo agent.
	m_SelectedAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[0]);
	for(unsigned i = 1; i < owner->GetMembers().size(); i++)
	{
		Agent* nextAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
		if(m_SelectedAgent->GetWeapon()->GetAmmoCount() > nextAgent->GetWeapon()->GetAmmoCount())
		{
			m_SelectedAgent = nextAgent;
		}
	}
	m_AmmoOnEnter = m_SelectedAgent->GetWeapon()->GetAmmoCount();

	//Find closest Ammo pickup.
	AmmoPickup* closestPickup = (AmmoPickup*)gEntities->GetEntity(owner->GetAmmoPickups()[0]);
	float distanceToClosestPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - closestPickup->GetPosition()));
	for(unsigned i = 1; i < owner->GetAmmoPickups().size(); i++)
	{
		AmmoPickup* nextPickup = (AmmoPickup*)gEntities->GetEntity(owner->GetAmmoPickups()[i]);
		float distanceToNextPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
		if((distanceToNextPickup < distanceToClosestPickup) && (nextPickup->IsActive()))
		{
			closestPickup = nextPickup;
			distanceToClosestPickup = distanceToNextPickup;
		}
	}
	m_SelectedPickup = closestPickup;
	//Move agent towards pickup.
	m_SelectedAgent->SetMoveTarget(m_SelectedPickup->GetPosition());
	gMessenger->AddMessage(-1, m_SelectedAgent->GetID(), Message::msg_Move, 0.0f);
}

void AgentTeamState_LowAmmo::Execute(AgentTeam* owner, float dt)
{
	//If the agent's ammo has gone up (by collecting a pickup), restart the state
	if(m_SelectedAgent->GetWeapon()->GetAmmoCount() > m_AmmoOnEnter)
		Enter(owner);

	//If Ammo pickup inactive, find a new pickup and send agent that way.
	if(!m_SelectedPickup->IsActive())
	{
			//Find closest Ammo pickup.
		AmmoPickup* closestPickup = (AmmoPickup*)gEntities->GetEntity(owner->GetAmmoPickups()[0]);
		float distanceToClosestPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - closestPickup->GetPosition()));
		for(unsigned i = 1; i < owner->GetAmmoPickups().size(); i++)
		{
			AmmoPickup* nextPickup = (AmmoPickup*)gEntities->GetEntity(owner->GetAmmoPickups()[i]);
			float distanceToNextPickup = D3DXVec3Length(&(m_SelectedAgent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
			if((distanceToNextPickup < distanceToClosestPickup) && (nextPickup->IsActive()))
			{
				closestPickup = nextPickup;
				distanceToClosestPickup = distanceToNextPickup;
			}
		}

		//Move agent towards pickup.
		m_SelectedAgent->SetMoveTarget(m_SelectedPickup->GetPosition());
		gMessenger->AddMessage(-1, m_SelectedAgent->GetID(), Message::msg_Move, 0.0f);
	}
}

void AgentTeamState_LowAmmo::Exit(AgentTeam* owner)
{

}

void AgentTeamState_LowAmmo::Render(AgentTeam* owner)
{
	//Rendering of agents handled by entity manager.
}

void AgentTeamState_Regroup::Enter(AgentTeam* owner)
{

}

void AgentTeamState_Regroup::Execute(AgentTeam* owner, float dt)
{
	//Update move targets
	Agent* firstAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[0]);
	for(unsigned i = 1; i < owner->GetMembers().size(); i++)
	{
		Agent* nextAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
		nextAgent->SetMoveTarget(firstAgent->GetPhysics()->GetPosition());
		gMessenger->AddMessage(-1, owner->GetMembers()[i], Message::msg_Move, 0.0f);
	}
}

void AgentTeamState_Regroup::Exit(AgentTeam* owner)
{

}

void AgentTeamState_Regroup::Render(AgentTeam* owner)
{

}

AgentTeamState_Regroup* AgentTeamState_Regroup::Instance()
{
	static AgentTeamState_Regroup instance;
	return &instance;
}

void AgentTeamState_Attack::Enter(AgentTeam* owner)
{

}

void AgentTeamState_Attack::Execute(AgentTeam* owner, float dt)
{
	if(owner->GetOpponent()->GetMembers().size() > 0)
	{
		Agent* firstEnemy = (Agent*)gEntities->GetEntity(owner->GetOpponent()->GetMembers()[0]);

		if(firstEnemy->GetHealth() <= 0.0f)
		{
			for(unsigned i = 0; i < owner->GetMembers().size(); i++)
			{
					gMessenger->AddMessage(-1, owner->GetMembers()[i], Message::msg_Stop, 0.0f);
			}
		}
		else
		{
			for(unsigned i = 0; i < owner->GetMembers().size(); i++)
			{
				Agent* nextAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
				nextAgent->SetMoveTarget(firstEnemy->GetPhysics()->GetPosition());
				gMessenger->AddMessage(-1, owner->GetMembers()[i], Message::msg_Attack, 0.0f);
			}
		}
	}
}

void AgentTeamState_Attack::Exit(AgentTeam* owner)
{

}

void AgentTeamState_Attack::Render(AgentTeam* owner)
{

}

AgentTeamState_Attack* AgentTeamState_Attack::Instance()
{
	static AgentTeamState_Attack instance;
	return &instance;
}

void AgentTeamState_Retreat::Enter(AgentTeam* owner)
{

}

void AgentTeamState_Retreat::Execute(AgentTeam* owner, float dt)
{
	if(owner->GetOpponent()->GetMembers().size() > 0)
	{
		//Flee the first enemy's position (since there is always a first agent).
		Agent* firstEnemy = (Agent*)gEntities->GetEntity(owner->GetOpponent()->GetMembers()[0]);
		if(firstEnemy)
		{
			for(unsigned i = 0; i < owner->GetMembers().size(); i++)
			{
				Agent* nextAgent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
				nextAgent->SetMoveTarget(firstEnemy->GetPhysics()->GetPosition());
				gMessenger->AddMessage(-1, owner->GetMembers()[i], Message::msg_Retreat, 0.0f);
			}
		}
	}
}

void AgentTeamState_Retreat::Exit(AgentTeam* owner)
{

}

void AgentTeamState_Retreat::Render(AgentTeam* owner)
{

}

AgentTeamState_Retreat* AgentTeamState_Retreat::Instance()
{
	static AgentTeamState_Retreat instance;
	return &instance;
}

void AgentTeamState_Global::Enter(AgentTeam* owner)
{

}

void AgentTeamState_Global::Execute(AgentTeam* owner, float dt)
{
	if((owner->GetOpponent()->GetMembers().size() != 0) && (owner->GetMembers().size() != 0))
	{
		if(owner->GetHealthState() == HealthLow)
			gMessenger->AddMessage(-1, owner->GetID(), msg_HealthLow, 0.0f);
		else if (owner->GetAmmoState() == AmmoLow)
			gMessenger->AddMessage(-1, owner->GetID(), msg_AmmoLow, 0.0f);
		else
			gMessenger->AddMessage(-1, owner->GetID(), msg_Attack, 0.0f);
	
		//Check for contact with pickups. Reset pickups and add ammo/health if contacted and usable.
		for(unsigned i = 0; i < owner->GetMembers().size(); i++)
		{
			Agent* agent = (Agent*)gEntities->GetEntity(owner->GetMembers()[i]);
	
			for(unsigned j = 0; j < owner->GetHealthPickups().size(); j++)
			{
				int nextID = owner->GetHealthPickups()[j];
				HealthPickup* nextPickup = (HealthPickup*)gEntities->GetEntity(nextID);
				float distanceToAgent = D3DXVec3Length(&(agent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
				if(distanceToAgent < (agent->GetPhysics()->GetContactRadius() + nextPickup->GetContactRadius()))
				{
					if(nextPickup->IsActive())
					{
							if(agent->GetHealth() < 100)
						{
							agent->AdjustHealth(nextPickup->GetAmount());
							gMessenger->AddMessage(-1, nextID, msg_Reset, 0.0);
						}
					}
				}
						
			}
			for(unsigned j = 0; j < owner->GetAmmoPickups().size(); j++)
			{
				int nextID = owner->GetAmmoPickups()[j];
				AmmoPickup* nextPickup = (AmmoPickup*)gEntities->GetEntity(nextID);
				float distanceToAgent = D3DXVec3Length(&(agent->GetPhysics()->GetPosition() - nextPickup->GetPosition()));
				if(distanceToAgent < (agent->GetPhysics()->GetContactRadius() + nextPickup->GetContactRadius()))
				{
					if(nextPickup->IsActive())
					{
						if(agent->GetWeapon()->GetAmmoCount() < agent->GetWeapon()->GetMaxAmmo())
						{
							agent->GetWeapon()->Reload(nextPickup->GetAmount());
							gMessenger->AddMessage(-1, nextID, msg_Reset, 0.0);
						}
					}
				}
			}
		}
	}
	else gMessenger->AddMessage(-1, owner->GetID(), msg_Stop, 0.0f);
}

void AgentTeamState_Global::Exit(AgentTeam* owner)
{

}

void AgentTeamState_Global::Render(AgentTeam* owner)
{

}

AgentTeamState_Global* AgentTeamState_Global::Instance()
{
	static AgentTeamState_Global instance;
	return &instance;
}
