#include "DirectInput.h"
#include "Defines.h"
#include "DirectX9App.h"

DInputFramework::~DInputFramework()
{
	Shutdown();
}

bool DInputFramework::KeyDown(char key)
{
	if(m_KeyboardState[key] & 0x80)
		return true;
	else
		return false;
}

bool DInputFramework::KeyReleased(char key)
{
	if((m_PrevKeyboardState[key] & 0x80) && !KeyDown(key))
		return true;
	else
		return false;
}

bool DInputFramework::KeyHeld(char key)
{
	if((m_PrevKeyboardState[key] & 0x80) && KeyDown(key))
		return true;
	else
		return false;
}

bool DInputFramework::MouseButtonDown(int button)
{
	if(m_MouseState.rgbButtons[button] & 0x80)
		return true;
	else
		return false;
}

bool DInputFramework::MouseButtonReleased(int button)
{
	if((m_PrevMouseState.rgbButtons[button] & 0x80) && !MouseButtonDown(button))
		return true;
	else
		return false;
}

bool DInputFramework::MouseButtonHeld(int button)
{
	if((m_PrevMouseState.rgbButtons[button] & 0x80) && MouseButtonDown(button))
		return true;
	else
		return false;
}

void DInputFramework::Init(HWND hWnd, HINSTANCE hInst)
{
	HR(DirectInput8Create(hInst, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&m_pInput, NULL));

	HR(m_pInput->CreateDevice(GUID_SysKeyboard, &m_pKeyboard, NULL));
	HR(m_pInput->CreateDevice(GUID_SysMouse, &m_pMouse, NULL));

	//Data formats
	HR(m_pKeyboard->SetDataFormat(&c_dfDIKeyboard));
	HR(m_pMouse->SetDataFormat(&c_dfDIMouse2));

	//Cooperative Levels
	HR(m_pKeyboard->SetCooperativeLevel(hWnd, KEYBOARD_COOP_FLAGS));
	HR(m_pMouse->SetCooperativeLevel(hWnd, MOUSE_COOP_FLAGS));

	//Aquire Devices
	HR(m_pKeyboard->Acquire());
	HR(m_pMouse->Acquire());

	//Initialize keyboard and mouse states
	for(int i = 0; i < 256; ++i)
	{
		m_PrevKeyboardState[i] = false;
	}
	ZeroMemory(m_KeyboardState, sizeof(m_KeyboardState));
	ZeroMemory(&m_MouseState, sizeof(m_MouseState));
	ZeroMemory(&m_PrevMouseState, sizeof(m_PrevMouseState));
	POINT p;
	GetCursorPos(&p);
	ScreenToClient(gApplication->GetMainWnd(), &p);
	m_MousePos.x = (float)p.x;
	m_MousePos.y = (float)p.y;
}

void DInputFramework::Update()
{
	//Update previous states.
	for(int i = 0; i < 256; ++i)
	{
		m_PrevKeyboardState[i] = m_KeyboardState[i];
	}

	m_PrevMouseState = m_MouseState;

	//Get keyboard input. If failed, zero out the keyboard state and reacquire the keyboard for next update.
	HRESULT hr = m_pKeyboard->GetDeviceState(sizeof(m_KeyboardState), (LPVOID)&m_KeyboardState);
	if(FAILED(hr))
	{
		ZeroMemory(m_KeyboardState, sizeof(m_KeyboardState));
		HR(m_pKeyboard->Acquire());
	}

	//Get mouse input. If failed, zero out mouse state and reacquire the mouse for next update.
	hr = m_pMouse->GetDeviceState(sizeof(DIMOUSESTATE2), &m_MouseState);
	if(FAILED(hr))
	{
		ZeroMemory(&m_MouseState, sizeof(m_MouseState));
		HR(m_pMouse->Acquire());
	}

	//Update mouse position
	POINT p;
	GetCursorPos(&p);
	ScreenToClient(gApplication->GetMainWnd(), &p);
	m_MousePos.x = (float)p.x;
	m_MousePos.y = (float)p.y;
}

void DInputFramework::Render()
{
	//Draw mouse position
	D3DXMATRIX worldMat;
	D3DXMatrixIdentity(&worldMat);
	gSprite->SetTransform(&worldMat);
	char buffer[64];

	sprintf_s(buffer, "X: %d\nY: %d", (int)gInput->GetMousePos().x, (int)gInput->GetMousePos().y);
	gApplication->GetDefaultFont()->DrawText(gSprite, buffer, -1, &gApplication->GetWindowRect(), DT_BOTTOM | DT_RIGHT | DT_NOCLIP, D3DCOLOR_ARGB(255,255,255,255));

	//Draw mouse cursor
}

void DInputFramework::Shutdown()
{
	//Unaquire input devices and release members
	SAFE_RELEASE(m_pInput);
	SAFE_RELEASE(m_pMouse);
	SAFE_RELEASE(m_pKeyboard);
	
}

DInputFramework* DInputFramework::Instance()
{
	static DInputFramework instance;
	return &instance;
}
