#include "EntityManager.h"
#include "Defines.h"

EntityManager* EntityManager::Instance()
{
	static EntityManager instance;
	return &instance;
}

BaseEntity* EntityManager::GetEntity(int ID)
{
	if(m_EntityMap.find(ID) != m_EntityMap.end())
	{
		return m_EntityMap[ID];
	}
	else return nullptr;
}

void EntityManager::AddEntity(BaseEntity* entity)
{
	if(m_EntityMap.find(entity->GetID()) == m_EntityMap.end())
	{
		m_EntityMap[entity->GetID()] = entity;
	}
}

void EntityManager::RemoveEntity(int ID)
{
	std::map<int, BaseEntity*>::const_iterator results = m_EntityMap.find(ID);
	if(results != m_EntityMap.end())
	{
		delete m_EntityMap[ID];
		m_EntityMap.erase(ID);
	}
}

void EntityManager::Update(float dt)
{
	std::map<int, BaseEntity*>::iterator itr;
	for(itr = m_EntityMap.begin(); itr != m_EntityMap.end(); ++itr)
	{
		itr->second->Update(dt);
	}
}

void EntityManager::Render()
{
	std::map<int, BaseEntity*>::iterator itr;
	for(itr = m_EntityMap.begin(); itr != m_EntityMap.end(); ++itr)
	{
		itr->second->Render();
	}
}

void EntityManager::Reset()
{
	std::map<int, BaseEntity*>::iterator itr;
	for(itr = m_EntityMap.begin(); itr != m_EntityMap.end(); ++ itr)
	{
		delete itr->second;
	}
	m_EntityMap.clear();
}