#include "GameStatePlaying.h"
#include "Defines.h"
#include "DirectX9App.h"
#include "DirectInput.h"
#include "D3DXMathFunctions.h"
#include "EntityGenerator.h"
#include "EntityManager.h"
#include "BaseEntity.h"
#include "AgentStates.h"
#include "Messages.h"
#include "AgentTeam.h"
#include "EntityGenerator.h"
#include "Pickups.h"
#include "DirectX9Game.h"
#include "GameStateEnded.h"


GameStatePlaying::GameStatePlaying():
	m_Obstacle1(D3DXVECTOR3(500,500,0)),
	m_Obstacle2(D3DXVECTOR3(200,650,0)),
	m_Obstacle3(D3DXVECTOR3(900,100,0)),
	m_Obstacle4(D3DXVECTOR3(650,250,0)),
	m_Obstacle5(D3DXVECTOR3(100,150,0))
{
	
}

GameStatePlaying::~GameStatePlaying()
{
	
}

GameStatePlaying* GameStatePlaying::Instance()
{
	static GameStatePlaying instance;
	return &instance;
}

void GameStatePlaying::Enter(DirectX9Game*)
{
	//Create Red Team
	int newID = gEntGenerate->CreateEntity(ent_Team);
	AgentTeam* redTeam = (AgentTeam*)gEntities->GetEntity(newID);
	m_AgentTeams.push_back(redTeam);

	//Create Blue Team
	newID = gEntGenerate->CreateEntity(ent_Team);
	AgentTeam* blueTeam = (AgentTeam*)gEntities->GetEntity(newID);
	blueTeam->SetColor(TeamColor::blue);
	blueTeam->SetOpponent(redTeam);
	redTeam->SetOpponent(blueTeam);
	m_AgentTeams.push_back(blueTeam);

	//Create pickups and set positions.
	newID = gEntGenerate->CreateEntity(ent_HealthPickup);
	HealthPickup* newHealthPickup = (HealthPickup*)gEntities->GetEntity(newID);
	newHealthPickup->SetPosition(D3DXVECTOR3(200,200,0));
	redTeam->AddHealthPickup(newID);
	blueTeam->AddHealthPickup(newID);

	newID = gEntGenerate->CreateEntity(ent_HealthPickup);
	newHealthPickup = (HealthPickup*)gEntities->GetEntity(newID);
	newHealthPickup->SetPosition(D3DXVECTOR3(800,600,0));
	redTeam->AddHealthPickup(newID);
	blueTeam->AddHealthPickup(newID);

	newID = gEntGenerate->CreateEntity(ent_AmmoPickup);
	AmmoPickup* newAmmoPickup = (AmmoPickup*)gEntities->GetEntity(newID);
	newAmmoPickup->SetPosition(D3DXVECTOR3(800,200,0));
	redTeam->AddAmmoPickup(newID);
	blueTeam->AddAmmoPickup(newID);

	newID = gEntGenerate->CreateEntity(ent_AmmoPickup);
	newAmmoPickup = (AmmoPickup*)gEntities->GetEntity(newID);
	newAmmoPickup->SetPosition(D3DXVECTOR3(250,650,0));
	redTeam->AddAmmoPickup(newID);
	blueTeam->AddAmmoPickup(newID);

	redTeam->Init();
	blueTeam->Init();
	if(redTeam->GetMembers().size() > 0)
	{
		for(unsigned i = 0; i < redTeam->GetMembers().size(); i++)
		{
			//Obstacle loading
			Agent* agent = (Agent*)gEntities->GetEntity(redTeam->GetMembers()[i]);
			agent->GetSteering()->AddObstacle(&m_Obstacle1);
			agent->GetSteering()->AddObstacle(&m_Obstacle2);
			agent->GetSteering()->AddObstacle(&m_Obstacle3);
			agent->GetSteering()->AddObstacle(&m_Obstacle4);
			agent->GetSteering()->AddObstacle(&m_Obstacle5);
		}
	}

	if(blueTeam->GetMembers().size() > 0)
	{
		for(unsigned i = 0; i < blueTeam->GetMembers().size(); i++)
		{
			//Obstacle loading
			Agent* agent = (Agent*)gEntities->GetEntity(blueTeam->GetMembers()[i]);
			agent->GetSteering()->AddObstacle(&m_Obstacle1);
			agent->GetSteering()->AddObstacle(&m_Obstacle2);
			agent->GetSteering()->AddObstacle(&m_Obstacle3);
			agent->GetSteering()->AddObstacle(&m_Obstacle4);
			agent->GetSteering()->AddObstacle(&m_Obstacle5);
		}
	}


	
}

void GameStatePlaying::Execute(DirectX9Game* owner, float dt)
{
	gEntities->Update(dt);
	for(unsigned i = 0; i < m_AgentTeams.size(); i++)
	{
		if(m_AgentTeams[i]->GetMembers().size() == 0)
			owner->GetFSM()->ChangeState(GameStateEnded::Instance());
	}
}

void GameStatePlaying::Render(DirectX9Game*)
{
	gEntities->Render();
	m_Obstacle1.Render();
	m_Obstacle2.Render();
	m_Obstacle3.Render();
	m_Obstacle4.Render();
	m_Obstacle5.Render();
		
}
void GameStatePlaying::Exit(DirectX9Game*)
{

}