#include "Physics.h"
#include "D3DXMathFunctions.h"
#include "DirectX9App.h"

void Physics::Init(D3DXVECTOR3 position, D3DXVECTOR3 velocity, float contactRadius, float maxAccel, float maxSpeed)
{
	m_Position = position;
	m_Velocity = velocity;
	m_Acceleration = D3DXZEROVEC3;
	m_Gravity = D3DXZEROVEC3;
	m_ContactRadius = contactRadius;
	m_MaxAccel = maxAccel;
	m_MaxSpeed = maxSpeed;
	m_FrictionForce = 100;
}


void Physics::Update(float dt)
{
	//Update position
	m_Position += m_Velocity * dt;

	//Update Acceleration
	m_Acceleration += m_Gravity;
	if(D3DXVec3Length(&m_Acceleration) > m_MaxAccel)
	{
		D3DXVec3Normalize(&m_Acceleration, &m_Acceleration);
		m_Acceleration *= m_MaxAccel;
	}

	//Apply friction force if there is a velocity
	if(D3DXVec3Length(&m_Velocity) > 0.1)
	{
		D3DXVECTOR3 frictionDirection;
		D3DXVec3Normalize(&frictionDirection, &m_Velocity);
		frictionDirection *= -m_FrictionForce;
		Accelerate(frictionDirection);
	}

	//Update velocity
	m_Velocity += m_Acceleration * dt;
	if(D3DXVec3Length(&m_Velocity) > m_MaxSpeed)
	{
		D3DXVec3Normalize(&m_Velocity, &m_Velocity);
		m_Velocity *= m_MaxSpeed;
	}

	//Update rotation
	m_Rotation = atan2(GetHeading().y, GetHeading().x);

	//Reset Acceleration
	m_Acceleration = D3DXZEROVEC3;

	//Clamp position to window
	if(m_Position.x > gApplication->GetWindowSize().x)
		m_Position.x = gApplication->GetWindowSize().x;
	if(m_Position.y > gApplication->GetWindowSize().y)
		m_Position.y = gApplication->GetWindowSize().y;
	if(m_Position.x < 0)
		m_Position.x = 0;
	if(m_Position.y < 0)
		m_Position.y = 0;
}


D3DXVECTOR3 Physics::GetHeading()
{
	D3DXVECTOR3 output;
	D3DXVec3Normalize(&output, &m_Velocity);
	return output;
}