#include "Rifle.h"
#include "Agent.h"
#include "D3DXMathFunctions.h"
#include "AgentTeam.h"
#include "EntityManager.h"

void Rifle::Fire()
{
	if(m_AmmoCount > 0)
	{
		if(m_FireCooldown <= 0)
		{
			Bullet newBullet(1.0f, 1000.0f, m_Owner->GetPhysics()->GetHeading(), m_Owner->GetPhysics()->GetPosition());
			m_Bullets.push_back(newBullet);
			m_AmmoCount--;
			m_FireCooldown = m_RateOfFire;
		}
	}
}

void Rifle::Update(float dt)
{
	//Update firing cooldown.
	if(m_FireCooldown > 0)
		m_FireCooldown -= dt;
	
	std::vector<Bullet> validBullets;  //List of bullets to retain this tick
	validBullets.clear();
	if(m_Bullets.size() > 0)
	{
		for(unsigned i = 0; i < m_Bullets.size(); i++)
		{
			m_Bullets[i].Update(dt);
			if(m_Owner->GetTeam()->GetOpponent()->GetMembers().size() != 0)
			{
				//Check contact with enemy agents
				for(unsigned j = 0; j < m_Owner->GetTeam()->GetMembers().size(); j++)
				{
					Agent* nextAgent = (Agent*)gEntities->GetEntity(m_Owner->GetTeam()->GetOpponent()->GetMembers()[j]);
					//If not the holder of this weapon check for contact.
					if (nextAgent)
				{
						//If distance between them less than contact radius, deal damage and add bullet for removal
							if(D3DXVec3Length(&(m_Bullets[i].GetPosition() - nextAgent->GetPhysics()->GetPosition())) < nextAgent->GetPhysics()->GetContactRadius() * 1.5)
						{
							nextAgent->AdjustHealth(-m_Damage);
							m_Bullets[i].Destroy();
						}
					}
				}
			}
			if(m_Bullets[i].GetLifetime() > 0.0f)
				validBullets.push_back(m_Bullets[i]);
		}
		//clear the bullet cache and restack
		m_Bullets.clear();
		m_Bullets = validBullets;
	}
	
	
	
}

void Rifle::Reload(int amount)
{
	m_AmmoCount += amount;
	if(m_AmmoCount > m_MaxAmmo)
		m_AmmoCount = m_MaxAmmo;
}

void Rifle::Render()
{
	if(m_Bullets.size() > 0)
	{
		for(unsigned i = 0; i < m_Bullets.size(); i++)
		{
			m_Bullets[i].Render();
		}
	}
}

void Bullet::Update(float dt)
{
	m_Lifetime -= dt;
	m_Position += (m_Direction * m_Speed * dt);
}

void Bullet::Render()
{
	D3DXMATRIX transMat;
	D3DXMatrixTranslation(&transMat, m_Position.x, m_Position.y, 0.0f);
	HR(gSprite->SetTransform(&transMat));
	HR(gSprite->Draw(m_Texture, 0, &D3DXZEROVEC3, &D3DXZEROVEC3, D3DCOLOR_ARGB(255,255,255,255)));
}