#pragma once
#include "State.h"
#include <cassert>

template <class t>
class StateMachine
{

protected:

	t*			m_Owner;
	//Current state. Proccessed on every update.
	State<t>*	m_CurrentState;
	//Last state. Allows reverting to previous state.
	State<t>*	m_PrevState;
	//Global state. Also processed on every update (for common actions that happen in any state).
	State<t>*	m_GlobalState;

public:
	StateMachine(t* owner, State<t>* startState, State<t>* startGlobalState, State<t>* startPrevState):
		m_Owner(owner),
		m_CurrentState(startState),
		m_GlobalState(startGlobalState),
		m_PrevState(startPrevState)
	{
		if(m_GlobalState)
			m_GlobalState->Enter(owner);
		if(m_CurrentState)
			m_CurrentState->Enter(owner);
	}

	//Accessors
	State<t>* GetCurrentState(){ return m_CurrentState; }
	State<t>* GetPrevState(){ return m_PrevState; }
	State<t>* GetGlobalState(){ return m_GlobalState;}

	//Modifiers
	void SetCurrentState(State<t>* newState)
	{
		m_CurrentState = newState;
		m_CurrentState->Enter(m_Owner);
	}
	void SetGlobalState(State<t>* newState)
	{
		m_GlobalState = newState;
		m_GlobalState->Enter(m_Owner);
	}

	void Update(float dt)
	{
		if(m_CurrentState)
			m_CurrentState->Execute(m_Owner, dt);
		if(m_GlobalState)
			m_GlobalState->Execute(m_Owner, dt);
	}

	void Render()
	{
		if(m_GlobalState)
			m_GlobalState->Render(m_Owner);
		if(m_CurrentState)
			m_CurrentState->Render(m_Owner);
	}

	void ChangeState(State<t>* state)
	{
		assert(state && "Attemping to change to a null state");
		m_PrevState = m_CurrentState;
		m_CurrentState->Exit(m_Owner);
		m_CurrentState = state;
		m_CurrentState->Enter(m_Owner);
	}

	void RevertState()
	{
		ChangeState(m_PrevState);
	}

	bool IsInState(State<t>* state)
	{
		if(state == m_CurrentState)
			return true;
		else
			return false;
	}
};





