#include "Steering.h"
#include "Agent.h"
#include "Defines.h"
#include "DirectX9App.h"
#include "AgentTeam.h"
#include "EntityManager.h"


Steering::Steering(Agent* owner):m_Owner(owner)
{
	m_Seeking = false;
	m_Arriving = false;
	m_Wandering = false;
	m_Avoiding = false;
	m_Fleeing = false;
	m_AvoidActive = false;
	D3DXIMAGE_INFO imageInfo;
	HR(D3DXCreateTextureFromFileEx(gD3DDevice, "Seek_Target_Blue.png", 0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_ARGB(255,0,0,255), &imageInfo, 0, &m_SeekTex));
	HR(D3DXCreateTextureFromFileEx(gD3DDevice, "Flee_Target_Blue.png", 0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_ARGB(255,0,0,255), &imageInfo, 0, &m_FleeTex));
	HR(D3DXCreateTextureFromFileEx(gD3DDevice, "Arrive_Target_Blue.png", 0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_ARGB(255,0,0,255), &imageInfo, 0, &m_ArriveTex));
	HR(D3DXCreateTextureFromFileEx(gD3DDevice, "Wander_Target_Blue.png", 0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, D3DCOLOR_ARGB(255,0,0,255), &imageInfo, 0, &m_WanderTex));
}

Steering::~Steering()
{
	SAFE_RELEASE(m_SeekTex);
	SAFE_RELEASE(m_FleeTex);
	SAFE_RELEASE(m_WanderTex);
	SAFE_RELEASE(m_ArriveTex);
}


void Steering::SetOwner(Agent* owner)
{
	m_Owner = owner;
}

void Steering::Seek(D3DXVECTOR3 target, float weight)
{
	m_Seeking = true;
	m_SeekTarget = target;
	m_SeekWeight = weight;
}

void Steering::SeekOff()
{
	m_Seeking = false;
}

bool Steering::IsSeeking()
{
	return m_Seeking;
}

void Steering::Flee(D3DXVECTOR3 target, float weight)
{
	m_Fleeing = true;
	m_FleeTarget = target;
	m_FleeWeight = weight;
}

void Steering::FleeOff()
{
	m_Fleeing = false;
}

bool Steering::IsFleeing()
{
	return m_Fleeing;
}

void Steering::Arrive(D3DXVECTOR3 target, float arriveRadius, float arriveSlowRadius, float arriveTime, float weight)
{
	m_Arriving = true;
	m_ArriveTarget = target;
	m_ArriveRadius = arriveRadius;
	m_ArriveSlowRadius = arriveSlowRadius;
	m_ArriveTime = arriveTime;
	m_ArriveWeight = weight;
}

void Steering::ArriveOff()
{
	m_Arriving = false;
}

bool Steering::IsArriving()
{
	return m_Arriving;
}

void Steering::Wander(float wanderRange, float wanderRadius, float wanderAmount, float weight)
{
	m_Wandering = true;
	m_WanderRange = wanderRange;
	m_WanderRadius = wanderRadius;
	m_WanderAmount = wanderAmount;
	m_WanderWeight = weight;
}

void Steering::WanderOff()
{
	m_Wandering = false;
}

bool Steering::IsWandering()
{
	return m_Wandering;
}

void Steering::AvoidObstacles(float avoidRange, float weight)
{
	m_Avoiding = true;
	m_AvoidanceRange = avoidRange;
	m_AvoidanceWeight = weight;
}

void Steering::AvoidObstaclesOff()
{
	m_Avoiding = false;
}

bool Steering::IsAvoiding()
{
	return m_Avoiding;
}

SteeringForce Steering::Calculate()
{
	SteeringForce output;
	m_AvoidActive = false;

	//Init Steering Force
	output.linear = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	
	//Temp variables for calculation
	D3DXVECTOR3 toTarget;
	//Calculate Seek Steering Force
	if(m_Seeking)
	{
		toTarget = m_SeekTarget - m_Owner->GetPhysics()->GetPosition();
		D3DXVec3Normalize(&toTarget, &toTarget);
		toTarget *= m_Owner->GetPhysics()->GetMaxAccel();
		output.linear += toTarget * m_SeekWeight;
	}

	//Calculate Flee Steering Force
	if(m_Fleeing)
	{
		toTarget = m_Owner->GetPhysics()->GetPosition() - m_FleeTarget;
		D3DXVec3Normalize(&toTarget, &toTarget);
		toTarget *= m_Owner->GetPhysics()->GetMaxAccel();
		output.linear += toTarget * m_FleeWeight;
	}

	//Calculate Arrive Steering Force
	if(m_Arriving)
	{
		toTarget = m_ArriveTarget - m_Owner->GetPhysics()->GetPosition();
		float distanceToTarget = D3DXVec3Length(&toTarget);
			
		//If we're outside the target radius, calculate the steering force
		if(distanceToTarget > m_ArriveRadius)
		{
			float targetSpeed;
			//Full speed if we're not at the slow range yet.
			if(distanceToTarget > m_ArriveSlowRadius)
			{
				targetSpeed = m_Owner->GetPhysics()->GetMaxSpeed();
			}
			//Otherwise, scale based on distance
			else
			{
				targetSpeed = m_Owner->GetPhysics()->GetMaxSpeed() * distanceToTarget / m_ArriveSlowRadius;
			}	
			//Generate the target velocity
			D3DXVec3Normalize(&toTarget, &toTarget);	//Velocity Direction
			toTarget *= targetSpeed;					//Velocity Magnitude
			
			//Translate this into target acceleration
			toTarget -= m_Owner->GetPhysics()->GetVelocity();
			toTarget *= ( 1.0f / m_ArriveTime );

			output.linear += toTarget * m_ArriveWeight;
		}
		
	}

	//Calculate Wander Steering Force
	if(m_Wandering)
	{
		//Set initial target (between +/- wander amount on each axis. 2D example, so Z is 0)
		int xSign =  rand()%2;
		int ySign =  rand()%2;
		if(xSign)
		{
			if(ySign)
				m_WanderTarget += D3DXVECTOR3(-rand()%100/100.0f, -rand()%100/100.0f, 0.0f) * m_WanderAmount;
			else
				m_WanderTarget += D3DXVECTOR3(-rand()%100/100.0f, -rand()%100/100.0f, 0.0f) * m_WanderAmount;
		}
		else
		{
			if(ySign)
				m_WanderTarget += D3DXVECTOR3(-rand()%100/100.0f, -rand()%100/100.0f, 0.0f) * m_WanderAmount;
			else
				m_WanderTarget += D3DXVECTOR3(-rand()%100/100.0f, -rand()%100/100.0f, 0.0f) * m_WanderAmount;
		}
		//Normalize vector back to unit circle
		D3DXVec3Normalize(&m_WanderTarget, &m_WanderTarget);
		//Fit to the assigned wander radius
		m_WanderTarget *= m_WanderRadius;
		//Find the center of the wander circle
		D3DXVECTOR3 wanderOrigin = m_Owner->GetPhysics()->GetPosition() + m_Owner->GetPhysics()->GetHeading() * m_WanderRange;
		//Add wander target to origin
		m_WanderTarget += wanderOrigin;
						
		//Seek wander target
		toTarget = m_WanderTarget - m_Owner->GetPhysics()->GetPosition();
		D3DXVec3Normalize(&toTarget, &toTarget);
		toTarget *= m_Owner->GetPhysics()->GetMaxAccel();
		output.linear += toTarget * m_WanderWeight;
	}

	//Claculate Avoid Steeing Force
	if(m_Avoiding)
	{
		if(D3DXVec3Length(&m_Owner->GetPhysics()->GetVelocity()) > 0.001f)
		{
			//Check against obstacles
			//Find the closest obstacle
			Obstacle* closestObstacle = nullptr;
			float distance = m_AvoidanceRange + m_Owner->GetPhysics()->GetContactRadius();
			float testDistance;		
			for(int i = 0; i < m_Obstacles.size(); ++i)
			{
				toTarget = m_Obstacles[i]->GetPosition() - m_Owner->GetPhysics()->GetPosition();
				testDistance = D3DXVec3Length(&toTarget);
				if(testDistance < distance)
				{
					closestObstacle = m_Obstacles[i];
				}
			}
			if(closestObstacle)
			{
				float distanceToTarget = D3DXVec3Length(&(m_Owner->GetPhysics()->GetPosition() - closestObstacle->GetPosition()));
				float sumOfContactRadii = m_Owner->GetPhysics()->GetContactRadius() + m_AvoidanceRange;
				float forceMultiplier = 1.0f;
				//If we're close to the target, start applying a repelling force.
				if(distanceToTarget < sumOfContactRadii)
				{
					forceMultiplier = (1-(distanceToTarget / sumOfContactRadii));
					D3DXVECTOR3 forceDirection = m_Owner->GetPhysics()->GetMaxAccel() * (m_Owner->GetPhysics()->GetPosition() - closestObstacle->GetPosition());
					D3DXVec3Normalize(&forceDirection, &forceDirection);
					output.linear += m_AvoidanceWeight * forceMultiplier * forceDirection * m_Owner->GetPhysics()->GetMaxAccel();
				}
			}

			//Check against other agents
			//Find the closest friendly agent
			Agent* closestAgent = nullptr;
			distance = m_AvoidanceRange + m_Owner->GetPhysics()->GetContactRadius();
			testDistance = 0.0f;
			for(int i = 0; i < m_Owner->GetTeam()->GetMembers().size(); ++i)
			{
				if(m_Owner->GetTeam()->GetMembers()[i] == m_Owner->GetID())
					continue;
				Agent* nextAgent = (Agent*)gEntities->GetEntity(m_Owner->GetTeam()->GetMembers()[i]);
				toTarget = nextAgent->GetPhysics()->GetPosition() - m_Owner->GetPhysics()->GetPosition();
				testDistance = D3DXVec3Length(&toTarget);
				if(testDistance < distance)
				{
					closestAgent = nextAgent;
				}
			}
			if(closestAgent)
			{
				float distanceToTarget = D3DXVec3Length(&(m_Owner->GetPhysics()->GetPosition() - closestAgent->GetPhysics()->GetPosition()));
				float sumOfContactRadii = m_Owner->GetPhysics()->GetContactRadius() + m_AvoidanceRange;
				float forceMultiplier = 1.0f;
				//If we're close to the target, start applying a repelling force.
				if(distanceToTarget < sumOfContactRadii)
				{
					forceMultiplier = (1-(distanceToTarget / sumOfContactRadii));
					D3DXVECTOR3 forceDirection = m_Owner->GetPhysics()->GetMaxAccel() * (m_Owner->GetPhysics()->GetPosition() - closestAgent->GetPhysics()->GetPosition());
					D3DXVec3Normalize(&forceDirection, &forceDirection);
					output.linear += m_AvoidanceWeight * forceMultiplier * forceDirection * m_Owner->GetPhysics()->GetMaxAccel();
				}
			}

			//Find the closest enemy agent
			closestAgent = nullptr;
			distance = m_AvoidanceRange + m_Owner->GetPhysics()->GetContactRadius();
			testDistance = 0.0f;
			for(int i = 0; i < m_Owner->GetTeam()->GetOpponent()->GetMembers().size(); ++i)
			{
				Agent* nextAgent = (Agent*)gEntities->GetEntity(m_Owner->GetTeam()->GetOpponent()->GetMembers()[i]);
				toTarget = nextAgent->GetPhysics()->GetPosition() - m_Owner->GetPhysics()->GetPosition();
				testDistance = D3DXVec3Length(&toTarget);
				if(testDistance < distance)
				{
					closestAgent = nextAgent;
				}
			}
			if(closestAgent)
			{
				float distanceToTarget = D3DXVec3Length(&(m_Owner->GetPhysics()->GetPosition() - closestAgent->GetPhysics()->GetPosition()));
				float sumOfContactRadii = m_Owner->GetPhysics()->GetContactRadius() + m_AvoidanceRange;
				float forceMultiplier = 1.0f;
				//If we're close to the target, start applying a repelling force.
				if(distanceToTarget < sumOfContactRadii)
				{
					forceMultiplier = (1-(distanceToTarget / sumOfContactRadii));
					D3DXVECTOR3 forceDirection = m_Owner->GetPhysics()->GetMaxAccel() * (m_Owner->GetPhysics()->GetPosition() - closestAgent->GetPhysics()->GetPosition());
					D3DXVec3Normalize(&forceDirection, &forceDirection);
					output.linear += m_AvoidanceWeight * forceMultiplier * forceDirection * m_Owner->GetPhysics()->GetMaxAccel();
				}
			}
		}
					
	}

	//Clamp output to max accel
	D3DXVec3Normalize(&output.linear, &output.linear);
	output.linear *= m_Owner->GetPhysics()->GetMaxAccel();
		
	//Output steering force
	return output;
}

void Steering::Render()
{
	D3DXMATRIX scaleMat, worldMat, transMat;
	if(m_Seeking)
	{
		D3DXMatrixScaling(&scaleMat, 0.7f, 0.7f, 0.7f);
		D3DXMatrixTranslation(&transMat, m_SeekTarget.x, m_SeekTarget.y, 0.0f);
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);
		HR(gSprite->SetTransform(&worldMat));
		HR(gSprite->Draw(m_SeekTex, 0, &D3DXVECTOR3(0.0f, 0.0f, 0.0f), 0, D3DCOLOR_ARGB(255,255,255,255)));
		HR(gSprite->Flush());
	}
	if(m_Fleeing)
	{
		D3DXMatrixScaling(&scaleMat, 0.7f, 0.7f, 0.7f);
		D3DXMatrixTranslation(&transMat, m_FleeTarget.x, m_FleeTarget.y, 0.0f);
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);
		HR(gSprite->SetTransform(&worldMat));
		HR(gSprite->Draw(m_FleeTex, 0, &D3DXVECTOR3(0.0f, 0.0f, 0.0f), 0, D3DCOLOR_ARGB(255,255,255,255)));
		HR(gSprite->Flush());
	}
	if(m_Wandering)
	{
		D3DXMatrixScaling(&scaleMat, 0.7f, 0.7f, 0.7f);
		D3DXMatrixTranslation(&transMat, m_WanderTarget.x, m_WanderTarget.y, 0.0f);
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);
		HR(gSprite->SetTransform(&worldMat));
		HR(gSprite->Draw(m_WanderTex, 0, &D3DXVECTOR3(0.0f, 0.0f, 0.0f), 0, D3DCOLOR_ARGB(255,255,255,255)));
		HR(gSprite->Flush());
	}
	if(m_Arriving)
	{
		D3DXMatrixScaling(&scaleMat, 0.7f, 0.7f, 0.7f);
		D3DXMatrixTranslation(&transMat, m_ArriveTarget.x, m_ArriveTarget.y, 0.0f);
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);
		HR(gSprite->SetTransform(&worldMat));
		HR(gSprite->Draw(m_ArriveTex, 0, &D3DXVECTOR3(0.0f, 0.0f, 0.0f), 0, D3DCOLOR_ARGB(255,255,255,255)));
		HR(gSprite->Flush());
	}
	if(m_Avoiding)
	{
		//Debug function. Draws "Avoiding" on the agent's target when an avoidance force is applied.
		//if(m_AvoidActive)
			//gApplication->GetDefaultFont()->DrawText(gSprite, "Avoiding", -1, 0, D3DFMT_UNKNOWN, D3DCOLOR_ARGB(255,0,0,0));
	}
}

void Steering::AddObstacle(Obstacle* in)
{
	if(in)
	{
		m_Obstacles.push_back(in);
	}
}