#pragma once

#include "d3dx9.h" //For D3D Math functions
#include <vector>
#include "Obstacle.h"

struct SteeringForce
{
	D3DXVECTOR3 linear;
};

class Agent;

class Steering
{
	std::vector<Obstacle*>		m_Obstacles;

	Agent*	m_Owner;
	//Target Textures
	IDirect3DTexture9*		m_SeekTex;
	IDirect3DTexture9*		m_FleeTex;
	IDirect3DTexture9*		m_ArriveTex;
	IDirect3DTexture9*		m_WanderTex;
	//Steering Weights
	D3DXVECTOR3		m_SeekTarget;
	D3DXVECTOR3		m_FleeTarget;
	D3DXVECTOR3		m_ArriveTarget;
	D3DXVECTOR3		m_WanderTarget;
	float			m_SeekWeight;
	float			m_ArriveWeight;
	float			m_WanderWeight;
	float			m_FleeWeight;
	float			m_AvoidanceWeight;
	
	//Control Variables
	bool			m_Seeking;
	bool			m_Fleeing;
	bool			m_Arriving;
	bool			m_Wandering;
	bool			m_Avoiding;
	bool			m_AvoidActive;			//Used for debugging purposes. Toggles the drawing of "Avoiding" next to the agents target if they have an avoid target. This is for debugging.

	//Wander Variables
	float			m_WanderRange;			//Distance from vehicle that the wander circle is projected.
	float			m_WanderRadius;			//Radius of the wander circle
	float			m_WanderAmount;			//Randomness of the wander target

	//Arrive variables
	float			m_ArriveRadius;			//Acceptable range from target to arrive at.
	float			m_ArriveSlowRadius;		//Radius to start slowing down at
	float			m_ArriveTime;			//Time it should take to arrive

	//Avoidance Variables
	float			m_AvoidanceRange;		//Range to check for obstacles
	
public:
	Steering(Agent* owner);
	~Steering();

	//Loads the textures for the targets
	void LoadTargets();

	void SetOwner(Agent* owner);

	//Seek Setter/Getter
	void Seek(D3DXVECTOR3 target, float weight);
	void SeekOff();
	bool IsSeeking();

	//Flee Setter/Getter
	void Flee(D3DXVECTOR3 target, float weight);
	void FleeOff();
	bool IsFleeing();

	//Arrive Setter/Getter
	void Arrive(D3DXVECTOR3 target, float arriveRadius, float arriveSlowRadius, float arriveTime, float weight);
	void ArriveOff();
	bool IsArriving();

	//Wander Setter/Getter
	void Wander(float wanderRange, float wanderRadius, float wanderAmount, float weight);
	void WanderOff();
	bool IsWandering();

	//Avoidance Setter/Getter
	void AvoidObstacles(float avoidRange, float weight);
	void AvoidObstaclesOff();
	bool IsAvoiding();

	//Steering Force Calculation
	SteeringForce Calculate();

	//Renders Seek/Flee/Wander/Arrive targets on screen.
	void Render();

	//Adds obstacles to avoid
	void AddObstacle(Obstacle* in);
};


